@@ -23,9 +23,9 @@ class Client(object):
     _MPUT = 'PUT'
     _MPOST = 'POST'
     _MDELETE = 'DELETE'
-    _comparison_conditions = ['prevValue', 'prevIndex', 'prevExist']
-    _read_options = ['recursive', 'wait', 'waitIndex', 'sorted']
-
+    _comparison_conditions = set(('prevValue', 'prevIndex', 'prevExist'))
+    _read_options = set(('recursive', 'wait', 'waitIndex', 'sorted', 'consistent'))
+    _del_conditions = set(('prevValue', 'prevIndex'))
     def __init__(
             self,
             host='127.0.0.1',
@@ -50,7 +50,7 @@ class Client(object):
             read_timeout (int):  max seconds to wait for a read.
 
             allow_redirect (bool): allow the client to connect to other nodes.
-
++
             protocol (str):  Protocol used to connect to etcd.
 
             cert (mixed):   If a string, the whole ssl client certificate;
@@ -207,6 +207,7 @@ class Client(object):
             key = "/{}".format(key)
         return key
 
+
     def write(self, key, value, ttl=None, dir=False, append=False, **kwdargs):
         """
         Writes the value for a key, possibly doing atomit Compare-and-Swap
@@ -239,9 +240,9 @@ class Client(object):
 
         """
         key = self._sanitize_key(key)
-        params = {
-            'value': value
-        }
+        params = {}
+        if value is not None:
+            params['value'] = value
 
         if ttl:
             params['ttl'] = ttl
@@ -268,6 +269,33 @@ class Client(object):
         response = self.api_execute(path, method, params=params)
         return self._result_from_response(response)
 
+    def update(self, obj):
+        """
+        Updates the value for a key atomically. Typical usage would be:
+
+        c = etcd.Client()
+        o = c.read("/somekey")
+        o.value += 1
+        c.update(o)
+
+        Args:
+            obj (etcd.EtcdResult):  The object that needs updating.
+
+        """
+        kwdargs = {
+            'dir': obj.dir,
+            'ttl': obj.ttl,
+            'prevExist': True
+            }
+
+        if not obj.dir:
+            # prevIndex on a dir causes a 'not a file' error. d'oh!
+            kwdargs['prevIndex'] = obj.modifiedIndex
+
+        return self.write(obj.key, obj.value, **kwdargs)
+
+
+
     def read(self, key, **kwdargs):
         """
         Returns the value of the key 'key'.
@@ -285,6 +313,8 @@ class Client(object):
 
             sorted (bool): Sort the output keys (alphanumerically)
 
+            timeout (int):  max seconds to wait for a read.
+
         Returns:
             client.EtcdResult (or an array of client.EtcdResult if a
             subtree is queried)
@@ -292,6 +322,8 @@ class Client(object):
         Raises:
             KeyValue:  If the key doesn't exists.
 
+            urllib3.exceptions.TimeoutError: If timeout is reached.
+
         >>> print client.get('/key').value
         'value'
 
@@ -306,21 +338,31 @@ class Client(object):
                 else:
                     params[k] = v
 
-        timeout = 'timeout' in kwdargs and kwdargs['timeout'] or None
+        timeout = kwdargs.get('timeout', None)
 
         response = self.api_execute(
             self.key_endpoint + key, self._MGET, params=params, timeout=timeout)
         return self._result_from_response(response)
 
-    def delete(self, key, recursive=None, dir=None):
+    def delete(self, key, recursive=None, dir=None, **kwdargs):
         """
         Removed a key from etcd.
 
         Args:
+
             key (str):  Key.
-            recursive (bool): if we want to recursively delete a directory, set it to true
+
+            recursive (bool): if we want to recursively delete a directory, set
+                              it to true
+
             dir (bool): if we want to delete a directory, set it to true
 
+            prevValue (str): compare key to this value, and swap only if
+                             corresponding (optional).
+
+            prevIndex (int): modify key only if actual modifiedIndex matches the
+                             provided one (optional).
+
         Returns:
             client.EtcdResult
 
@@ -339,8 +381,12 @@ class Client(object):
         if dir is not None:
             kwds['dir'] = dir and "true" or "false"
 
+        for k in self._del_conditions:
+            if k in kwdargs:
+                kwds[k] = kwdargs[k]
+
         response = self.api_execute(
-            self.key_endpoint + key, self._MDELETE, kwds)
+            self.key_endpoint + key, self._MDELETE, params=kwds)
         return self._result_from_response(response)
 
     # Higher-level methods on top of the basic primitives
@@ -415,12 +461,16 @@ class Client(object):
 
             index (int): Index to start from.
 
+            timeout (int):  max seconds to wait for a read.
+
         Returns:
             client.EtcdResult
 
         Raises:
             KeyValue:  If the key doesn't exists.
 
+            urllib3.exceptions.TimeoutError: If timeout is reached.
+
         >>> print client.watch('/key').value
         'value'
 
@@ -430,7 +480,7 @@ class Client(object):
         else:
             return self.read(key, wait=True, timeout=timeout)
 
-    def ethernal_watch(self, key, index=None):
+    def eternal_watch(self, key, index=None):
         """
         Generator that will yield changes from a key.
         Note that this method will block forever until an event is generated.
@@ -442,7 +492,7 @@ class Client(object):
         Yields:
             client.EtcdResult
 
-        >>> for event in client.ethernal_watch('/subcription_key'):
+        >>> for event in client.eternal_watch('/subcription_key'):
         ...     print event.value
         ...
         value1
@@ -465,13 +515,12 @@ class Client(object):
 
     def _result_from_response(self, response):
         """ Creates an EtcdResult from json dictionary """
-        # TODO: add headers we obtained from the http respose to the etcd
-        # result.
         try:
             res = json.loads(response.data.decode('utf-8'))
             r = etcd.EtcdResult(**res)
             if response.status == 201:
                 r.newKey = True
+            r.parse_headers(response)
             return r
         except Exception as e:
             raise etcd.EtcdException(
@@ -484,7 +533,7 @@ class Client(object):
         except IndexError:
             raise etcd.EtcdException('No more machines in the cluster')
 
-    def api_execute(self, path, method,  params=None, timeout=None):
+    def api_execute(self, path, method, params=None, timeout=None):
         """ Executes the query. """
 
         some_request_failed = False
