@@ -103,28 +103,44 @@ class TestSimple(EtcdIntegrationTest):
         except KeyError as e:
             pass
 
+    def test_update(self):
+        """INTEGRATION: update a value"""
+        self.client.set('/foo', 3)
+        c = self.client.get('/foo')
+        c.value = int(c.value) + 3
+        self.client.update(c)
+        newres = self.client.get('/foo')
+        self.assertEquals(newres.value, u'6')
+        self.assertRaises(ValueError, self.client.update, c)
+
     def test_retrieve_subkeys(self):
         """ INTEGRATION: retrieve multiple subkeys """
         set_result = self.client.write('/subtree/test_set', 'test-key1')
         set_result = self.client.write('/subtree/test_set1', 'test-key2')
         set_result = self.client.write('/subtree/test_set2', 'test-key3')
         get_result = self.client.read('/subtree', recursive=True)
-        result = [subkey.value for subkey in get_result.children]
+        result = [subkey.value for subkey in get_result.leaves]
         self.assertEquals(['test-key1', 'test-key2', 'test-key3'], result)
 
+    def test_directory_ttl_update(self):
+        """ INTEGRATION: should be able to update a dir TTL """
+        self.client.write('/dir', None, dir=True, ttl=30)
+        res = self.client.write('/dir', None, dir=True, ttl=31, prevExist=True)
+        self.assertEquals(res.ttl, 31)
+        res = self.client.get('/dir')
+        res.ttl = 120
+        new_res = self.client.update(res)
+        self.assertEquals(new_res.ttl, 120)
+
+
 
 class TestErrors(EtcdIntegrationTest):
 
     def test_is_not_a_file(self):
-        """ INTEGRATION: try to write  value to a directory """
+        """ INTEGRATION: try to write  value to an existing directory """
 
-        set_result = self.client.set('/directory/test-key', 'test-value')
-
-        try:
-            get_result = self.client.set('/directory', 'test-value')
-            assert False
-        except KeyError as e:
-            pass
+        self.client.set('/directory/test-key', 'test-value')
+        self.assertRaises(KeyError, self.client.set, '/directory', 'test-value')
 
     def test_test_and_set(self):
         """ INTEGRATION: try test_and_set operation """
@@ -136,15 +152,14 @@ class TestErrors(EtcdIntegrationTest):
             'test-value',
             'old-test-value')
 
-        try:
-            set_result = self.client.test_and_set(
-                '/test-key',
-                'new-value',
-                'old-test-value')
+        self.assertRaises(ValueError, self.client.test_and_set, '/test-key', 'new-value', 'old-test-value')
 
-            assert False
-        except ValueError as e:
-            pass
+    def test_creating_already_existing_directory(self):
+        """ INTEGRATION: creating an already existing directory without
+        `prevExist=True` should fail """
+        self.client.write('/mydir', None, dir=True)
+
+        self.assertRaises(KeyError, self.client.write, '/mydir', None, dir=True)
 
 
 class TestClusterFunctions(EtcdIntegrationTest):
@@ -347,7 +362,7 @@ class TestWatch(EtcdIntegrationTest):
         def watch_value(key, queue):
             c = etcd.Client(port=6001)
             for i in range(0, 3):
-                event = next(c.ethernal_watch(key)).value
+                event = next(c.eternal_watch(key)).value
                 queue.put(event)
 
         changer = multiprocessing.Process(
@@ -385,7 +400,7 @@ class TestWatch(EtcdIntegrationTest):
 
         def watch_value(key, index, queue):
             c = etcd.Client(port=6001)
-            iterevents = c.ethernal_watch(key, index=index)
+            iterevents = c.eternal_watch(key, index=index)
             for i in range(0, 3):
                 queue.put(next(iterevents).value)
 
@@ -471,16 +486,8 @@ class TestAuthenticatedAccess(EtcdIntegrationTest):
         client = etcd.Client(
             protocol='https', port=6001, ca_cert=self.ca2_cert_path)
 
-        try:
-            set_result = client.set('/test_set', 'test-key')
-            assert False
-        except urllib3.exceptions.SSLError as e:
-            assert True
-
-        try:
-            get_result = client.get('/test_set')
-        except urllib3.exceptions.SSLError as e:
-            assert True
+        self.assertRaises(urllib3.exceptions.SSLError, client.set, '/test-set', 'test-key')
+        self.assertRaises(urllib3.exceptions.SSLError, client.get, '/test-set')
 
     def test_get_set_authenticated(self):
         """ INTEGRATION: set/get a new value authenticated """
