@@ -35,7 +35,76 @@ class TestClientApiBase(unittest.TestCase):
     def _mock_exception(self, exc, msg):
         self.client.api_execute = mock.Mock(side_effect=exc(msg))
 
+
+class TestClientApiInternals(TestClientApiBase):
+
+    def test_read_default_timeout(self):
+        """ Read timeout set to the default """
+        d = {
+            u'action': u'get',
+            u'node': {
+                u'modifiedIndex': 190,
+                u'key': u'/testkey',
+                u'value': u'test'
+            }
+        }
+        self._mock_api(200, d)
+        res = self.client.read('/testkey')
+        self.assertEqual(self.client.api_execute.call_args[1]['timeout'], None)
+
+    def test_read_custom_timeout(self):
+        """ Read timeout set to the supplied value """
+        d = {
+            u'action': u'get',
+            u'node': {
+                u'modifiedIndex': 190,
+                u'key': u'/testkey',
+                u'value': u'test'
+            }
+        }
+        self._mock_api(200, d)
+        self.client.read('/testkey', timeout=15)
+        self.assertEqual(self.client.api_execute.call_args[1]['timeout'], 15)
+
+    def test_read_no_timeout(self):
+        """ Read timeout disabled """
+        d = {
+            u'action': u'get',
+            u'node': {
+                u'modifiedIndex': 190,
+                u'key': u'/testkey',
+                u'value': u'test'
+            }
+        }
+        self._mock_api(200, d)
+        self.client.read('/testkey', timeout=0)
+        self.assertEqual(self.client.api_execute.call_args[1]['timeout'], 0)
+
+    def test_write_no_params(self):
+        """ Calling `write` without a value argument will omit the `value` from
+        the API call params """
+        d = {
+            u'action': u'set',
+            u'node': {
+                u'createdIndex': 17,
+                u'dir': True,
+                u'key': u'/newdir',
+                u'modifiedIndex': 17
+            }
+        }
+        self._mock_api(200, d)
+        self.client.write('/newdir', None, dir=True)
+        self.assertEquals(self.client.api_execute.call_args,
+            (('/v2/keys/newdir', 'PUT'), dict(params={'dir': 'true'})))
+
+
+
 class TestClientApiInterface(TestClientApiBase):
+    """
+    All tests defined in this class are executed also in TestClientRequest.
+
+    If a test should be run only in this class, please override the method there.
+    """
 
     def test_machines(self):
         """ Can request machines """
@@ -67,6 +136,25 @@ class TestClientApiInterface(TestClientApiBase):
         res = self.client.write('/testkey', 'test')
         self.assertEquals(res, etcd.EtcdResult(**d))
 
+    def test_update(self):
+        """Can update a result."""
+        d = {u'action': u'set',
+             u'node': {
+                 u'expiration': u'2013-09-14T00:56:59.316195568+02:00',
+                 u'modifiedIndex': 6,
+                 u'key': u'/testkey',
+                 u'ttl': 19,
+                 u'value': u'test'
+                 }
+             }
+        self._mock_api(200,d)
+        res = self.client.get('/testkey')
+        res.value = 'ciao'
+        d['node']['value'] = 'ciao'
+        self._mock_api(200,d)
+        newres = self.client.update(res)
+        self.assertEquals(newres.value, 'ciao')
+
     def test_newkey(self):
         """ Can set a new value """
         d = {
@@ -84,6 +172,8 @@ class TestClientApiInterface(TestClientApiBase):
         d['node']['newKey'] = True
         self.assertEquals(res, etcd.EtcdResult(**d))
 
+
+
     def test_not_found_response(self):
         """ Can handle server not found response """
         self._mock_api(404, 'Not found')
@@ -398,3 +488,25 @@ class TestClientRequest(TestClientApiInterface):
 
     def test_in(self):
         pass
+
+    def test_update_fails(self):
+        """ Non-atomic updates fail """
+        d = {u'action': u'set',
+             u'node': {
+                u'expiration': u'2013-09-14T00:56:59.316195568+02:00',
+                u'modifiedIndex': 6,
+                u'key': u'/testkey',
+                u'ttl': 19,
+                u'value': u'test'
+                }
+             }
+        res = etcd.EtcdResult(**d)
+
+        error = {
+            "errorCode":101,
+            "message":"Compare failed",
+            "cause":"[ != bar] [7 != 6]",
+            "index":6}
+        self._mock_api(412, error)
+        res.value = 'bar'
+        self.assertRaises(ValueError, self.client.update, res)
